@@ -18,6 +18,7 @@ def validate_nodes(
     dependents,
     temporal_dimension_name,
     bands_dimension_name,
+    user_defined_processes,
 ):
     valid_subgraphs = []
     all_nodes_valid = True
@@ -34,7 +35,16 @@ def validate_nodes(
             valid = False
 
         try:
-            node = Node(node_id, process_id, arguments, [], dependency_graph[node_id], dependents[node_id], 0)
+            node = Node(
+                node_id,
+                process_id,
+                arguments,
+                [],
+                dependency_graph[node_id],
+                dependents[node_id],
+                0,
+                user_defined_processes=user_defined_processes,
+            )
         except ProcessDefinitionMissing as e:
             valid = False
 
@@ -48,7 +58,9 @@ def validate_nodes(
     return all_nodes_valid, valid_subgraphs
 
 
-def check_validity_and_subgraphs(process_graph, temporal_dimension_name, bands_dimension_name):
+def check_validity_and_subgraphs(
+    process_graph, temporal_dimension_name, bands_dimension_name, user_defined_processes={}
+):
     dependency_graph = get_dependencies(process_graph)
     dependents = get_dependents(dependency_graph)
     execution_order = get_execution_order(dependency_graph, dependents)
@@ -59,24 +71,31 @@ def check_validity_and_subgraphs(process_graph, temporal_dimension_name, bands_d
         dependents,
         temporal_dimension_name,
         bands_dimension_name,
+        user_defined_processes,
     )
 
 
 def convert_from_process_graph(
     process_graph,
+    user_defined_processes={},
     n_output_bands=1,
     sample_type="FLOAT32",
     units=None,
     bands_dimension_name="bands",
     temporal_dimension_name="t",
     encode_result=True,
+    bands_metadata=[],
 ):
     all_nodes_valid, subgraphs = check_validity_and_subgraphs(
-        process_graph, temporal_dimension_name, bands_dimension_name
+        process_graph, temporal_dimension_name, bands_dimension_name, user_defined_processes=user_defined_processes
     )
     if all_nodes_valid:
         nodes, input_bands, initial_data_name = generate_nodes_from_process_graph(
-            process_graph, bands_dimension_name, temporal_dimension_name, level=1
+            process_graph,
+            bands_dimension_name,
+            temporal_dimension_name,
+            user_defined_processes=user_defined_processes,
+            level=1,
         )
         evalscript = Evalscript(
             input_bands,
@@ -86,6 +105,7 @@ def convert_from_process_graph(
             sample_type=sample_type,
             units=units,
             encode_result=encode_result,
+            bands_metadata=bands_metadata,
         )
         output_dimensions = evalscript.determine_output_dimensions()
         evalscript.set_output_dimensions(output_dimensions)
@@ -102,6 +122,7 @@ def convert_from_process_graph(
                 subgraph["graph"],
                 bands_dimension_name,
                 temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
                 level=1,
             )
             evalscript = Evalscript(
@@ -112,6 +133,7 @@ def convert_from_process_graph(
                 sample_type=sample_type,
                 units=units,
                 encode_result=encode_result,
+                bands_metadata=bands_metadata,
             )
             output_dimensions = evalscript.determine_output_dimensions()
             evalscript.set_output_dimensions(output_dimensions)
@@ -124,7 +146,9 @@ def convert_from_process_graph(
         return evalscripts
 
 
-def generate_nodes_from_process_graph(process_graph, bands_dimension_name, temporal_dimension_name, level=1):
+def generate_nodes_from_process_graph(
+    process_graph, bands_dimension_name, temporal_dimension_name, user_defined_processes={}, level=1
+):
     dependency_graph = get_dependencies(process_graph)
     dependents = get_dependents(dependency_graph)
     execution_order = get_execution_order(dependency_graph, dependents)
@@ -135,6 +159,7 @@ def generate_nodes_from_process_graph(process_graph, bands_dimension_name, tempo
         dependents,
         temporal_dimension_name,
         bands_dimension_name,
+        user_defined_processes,
     )
 
     nodes = []
@@ -157,23 +182,66 @@ def generate_nodes_from_process_graph(process_graph, bands_dimension_name, tempo
                 arguments["reducer"]["process_graph"],
                 bands_dimension_name,
                 temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
                 level=level + 1,
             )
-        elif process_id == "apply":
+        elif process_id == "apply" or process_id == "apply_dimension":
             child_nodes, _, _ = generate_nodes_from_process_graph(
                 arguments["process"]["process_graph"],
                 bands_dimension_name,
                 temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
                 level=level + 1,
             )
         elif process_id == "count":
-            if "condition" in arguments and arguments["condition"] is not None: 
+            if "condition" in arguments and arguments["condition"] is not None:
                 child_nodes, _, _ = generate_nodes_from_process_graph(
                     arguments["condition"]["process_graph"],
                     bands_dimension_name,
                     temporal_dimension_name,
+                    user_defined_processes=user_defined_processes,
                     level=level + 1,
                 )
+        elif process_id == "array_apply":
+            child_nodes, _, _ = generate_nodes_from_process_graph(
+                arguments["process"]["process_graph"],
+                bands_dimension_name,
+                temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
+                level=level + 1,
+            )
+        elif process_id == "array_filter":
+            child_nodes, _, _ = generate_nodes_from_process_graph(
+                arguments["condition"]["process_graph"],
+                bands_dimension_name,
+                temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
+                level=level + 1,
+            )
+        elif process_id == "aggregate_temporal_period":
+            child_nodes, _, _ = generate_nodes_from_process_graph(
+                arguments["reducer"]["process_graph"],
+                bands_dimension_name,
+                temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
+                level=level + 1,
+            )
+        elif process_id == "aggregate_temporal":
+            child_nodes, _, _ = generate_nodes_from_process_graph(
+                arguments["reducer"]["process_graph"],
+                bands_dimension_name,
+                temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
+                level=level + 1,
+            )
+        elif process_id in user_defined_processes:
+            child_nodes, _, _ = generate_nodes_from_process_graph(
+                user_defined_processes[process_id],
+                bands_dimension_name,
+                temporal_dimension_name,
+                user_defined_processes=user_defined_processes,
+                level=level + 1,
+            )
         node = Node(
             node_id,
             process_id,
@@ -182,6 +250,7 @@ def generate_nodes_from_process_graph(process_graph, bands_dimension_name, tempo
             dependency_graph[node_id],
             dependents[node_id],
             level,
+            user_defined_processes=user_defined_processes,
         )
         nodes.append(node)
     return nodes, input_bands, initial_data_name
